//matrix.h
#ifndef MATRIX_H
#define MATRIX_H
#include "vector.h"

class Matrix{
  // class members
  double** pA; 
  int nRows, nCols;
  void memoryCheck();
  void memoryCheck(int i);

 public:
  //constructors and destructor
  Matrix(int r, int c, double** pa);//default constructor
  Matrix(int r = 0, int c = 0, double a = 0);
  Matrix(const Matrix& A);// copy constructor
  ~Matrix();//destructor

  // overloaded operators
  Matrix& operator=(const Matrix& B);
  Matrix operator+(const Matrix& B);
  Matrix& operator+=(const Matrix& B);
  Matrix operator-(const Matrix& B);
  Matrix& operator-=(const Matrix& B);
  Matrix operator*(const Matrix& B); 
  Matrix operator*(double b);//right scalar multiplication
  Vector operator*(Vector v);//matrix-vector multiplication
  double* operator[](int i){return pA[i];}

  // matrix operations
  Matrix trans();
  
  // solution of a linear system
  Matrix cholInv();
  Vector forward(Vector& RHS);
  Vector backward(Vector& RHS);

  // utilities and accessors
  void printMatrix();
  void vecToMat(int j, Vector& v);//insert Vector into jth column
  Vector matToVec(int j);//make jth column into a Vector
  int getNRows(){return nRows;}
  int getNCols(){return nCols;}

  //friend functions
  friend Matrix operator*(Vector& v1, Vector& v2);//outer product
  friend Matrix operator*(double, const Matrix& A);//left scalar multiplication
};

Matrix operator*(Vector& v1, Vector& v2);//vector outer product
Matrix operator*(double, const Matrix& A);//left scalar multiplication
#endif
