//vector.cpp
#include "vector.h"
#include <iostream>
#include <cstdlib>

using std::cout; using std::endl;

//constructors and destructors

Vector::Vector(int r, double* pa){
  nRows = r;
  pA = new double[nRows];
  memoryCheck();
      
  for(int i = 0; i < nRows; i++)
    pA[i] = pa[i];
}


Vector::Vector(int r, double b){
  nRows = r;
  pA = new double[nRows];
  memoryCheck();
      
  for(int i = 0; i < nRows; i++)
    pA[i] = b;
}

Vector::Vector(double* pa, int low, int up){
  if(up <= low){
    cout << "The lower index must be strictly smaller that the upper!" << endl;
    exit(1);
  }
  nRows = up - low + 1;
  pA = new double[nRows];
  for(int i = 0; i < nRows; i++)
    pA[i] = pa[i + low];
}

Vector::Vector(int r, int i){
  nRows = r;
  pA = new double[nRows];
  memoryCheck();
      
  for(int j = 0; j < nRows; j++){
    if(j == i) pA[j] = 1.;
    else pA[j] = 0.;
  }
}

//copy constructor
Vector::Vector(const Vector& v){
  nRows = v.nRows;
  pA = new double[nRows];
  memoryCheck();
      
  for(int i = 0; i < nRows; i++)
    pA[i] = v.pA[i];
}    


Vector::~Vector(){
  if(pA !=0)
    delete[] pA;
}

double Vector::dotProd(const Vector& v){
  double dot = 0;
  for(int i = 0; i < nRows; i++)
    dot += v.pA[i]*pA[i];
  return dot;
}

//overloaded operators

Vector& Vector::operator=(const Vector& v){

  if(this == &v) // avoid self assignment
    return *this;
  
  if(pA != 0){
    delete[] pA;
  }
    
  // define/redefine object's members
  nRows = v.nRows; 
  pA = new double[nRows];
  memoryCheck();
      
  for(int i = 0; i < nRows; i++)
    pA[i] = v.pA[i];

  return *this;
}    

Vector& Vector::operator-=(const Vector& v){
  if(nRows != v.nRows){
    cout << "Bad row dimensions in -=!" << endl;
    exit(1);
  }
  for(int i = 0; i < nRows; i++)
    pA[i] -= v.pA[i];
  
  return *this;
}

Vector Vector::operator*(double b){
  Vector temp(nRows);
  for(int i = 0; i < nRows; i++)
    temp.pA[i]=b*pA[i];
  
  return temp;
}

//utilities

void Vector::printVec(){
  for(int i = 0; i < nRows; i++)
    cout <<  pA[i] <<  endl;
}

void Vector::memoryCheck(){
  if(pA == 0){
    cout << "Memory allocation for pA failed" << endl;
    exit(1);
  }
}

Vector operator*(double b, const Vector& v){
    Vector B = v;
    return B*b;
}
